use std::io::prelude::*;
use crate::conf;
use std::sync::{Arc, RwLock};

const OP_ADD: u8 = 0;
const OP_DEL: u8 = 1;
const OP_EXIT: u8 = 2;

const RE_ADD: u8 = 0;
const RE_MOD: u8 = 1;

const RE_OK: u8 = 0;
const RE_KO: u8 = 1;

pub struct Conexion<TSocket: Read + Write> {
    conf: Arc<RwLock<conf::Config>>,
    stream: TSocket,
    buf:[u8; 256],
    exit: bool,
}

impl<TSocket: Read + Write> Conexion<TSocket> {
    pub fn new(conf: Arc<RwLock<conf::Config>>, stream: TSocket) -> Self {
        Self{
                    conf: conf,
                    stream: stream,
                    buf: [1; 256],
                    exit: false,
        }
    }

    pub fn process_reques(&mut self) {
        let mut state: [u8; 1] = [1; 1];
        while !self.exit {
        match self.stream.read_exact(&mut state){
            Ok(len) => self.check_state(state[0]),
            Err(e) => {self.exit=true; println!("Fatal: {}", e);},
        }
        }

    }

    fn read_domain_info(&mut self) ->
            Result<conf::ServerData, std::io::Error>{
        let mut len = self.stream.read(&mut self.buf)?;
        let end_domain = (self.buf[0] + 1) as usize;
        let start_ip = end_domain + 1 as usize;
        let end_ip = self.buf[end_domain] as usize + start_ip;
        let domain = String::from_utf8(self.buf[1 .. end_domain].to_vec()).unwrap();
        let ip = String::from_utf8(self.buf[start_ip .. end_ip].to_vec()).unwrap();
        Ok(conf::ServerData{
            domain: domain,
            ip: ip,
        })
    }

    fn read_unique_info(&mut self) ->
        Result<String, std::io::Error>{
        self.stream.read_exact(&mut self.buf[0 .. 1])?;
        let len = self.buf[0] as usize;
        self.stream.read_exact(&mut self.buf[0 .. len])?;
        Ok(String::from_utf8(self.buf[0 .. len].to_vec()).unwrap())
    }

    fn check_state(&mut self, state: u8) {
        match state{
            OP_ADD => {
                match self.read_domain_info() {
                    Ok(sd) => {
                        match self.conf.write().unwrap().add(sd) {
                            true => self.stream.write(&[RE_MOD]),
                            false => self.stream.write(&[RE_ADD]),
                        };
                        self.conf.read().unwrap().flush();
                    },
                    Err(e) => {self.exit=true; println!("e: {}", e);},
                }
            },

            OP_DEL => match self.read_unique_info(){
                Ok(domain) => {
                    match self.conf.write().unwrap().del(domain){
                        true => {
                            self.stream.write(&[RE_OK]);
                        },
                        false => {self.stream.write(&[RE_KO]);},
                    };
                    self.conf.read().unwrap().flush();
                }
                Err(e) => {self.exit=true; println!("e: {}", e);},
            },
            OP_EXIT => self.exit=true,
            _=> println!("no recognice option: {}", state),
        }
    }
}
