use std::net::TcpListener;
use std::net::TcpStream;
use std::os::unix::net::UnixListener;
use std::os::unix::net::UnixStream;
use std::io::{Error, ErrorKind};
use std::io::prelude::*;

pub const TCP_LIS : u8 = 0;
pub const UNIX_LIS : u8 = 1;

pub trait NewTrait: std::io::Read + std::io::Write + Send {}
impl NewTrait for TcpStream {}
impl NewTrait for UnixStream {}

pub struct GenericListener {
    tcp_lis : Option<TcpListener>,
    unix_lis : Option<UnixListener>,
    type_lis : u8,
}

impl GenericListener {
    fn new_tcp(listener: TcpListener) -> Self{
        GenericListener{
            tcp_lis: Some(listener),
            unix_lis: None,
            type_lis: TCP_LIS,
        }
    }

    fn new_unix(listener: UnixListener) -> Self{
        GenericListener{
            tcp_lis: None,
            unix_lis: Some(listener),
            type_lis: UNIX_LIS,
        }
    }

    pub fn bind(address: String, type_lis: u8) -> Result<Self, String>{
        let ret = match type_lis {
            TCP_LIS => Self::new_tcp(TcpListener::bind(address).unwrap()),
            UNIX_LIS => Self::new_unix(UnixListener::bind(address).unwrap()),
            _ => return Err("No valid option".to_string()),
        };
        Ok(ret)
    }



    pub fn accept(&self) -> Result<Box<dyn NewTrait>,Error> {
        match self.type_lis {
            TCP_LIS => Ok(Box::new(self.tcp_lis.as_ref().unwrap().accept()?.0)),
            UNIX_LIS => Ok(Box::new(self.unix_lis.as_ref().unwrap().accept()?.0)),
            _=> Err(Error::new(ErrorKind::Other, "Unexpected type"))
        }

    }

}

